//-	---------------------------------------------------------------------------
//-	Name: 		InPlaceCheckbox.cpp
//-
//-	Purpose:	This file provides the implementation of the class
//-				CGridCheckBox.
//-
//-	Revisions:
//-	Date		Author		Change Summary
//-	---------	------		---------------------------------------------------
//-	09-30-99	Mklombardi@ Created & Released.
//-             netzero,net 
//-
//-	Notes:		None.
//-
//- Copyright : This software is NOT copyrighted.
// This code may be used in compiled form in any way you desire. This
// file may be redistributed unmodified by any means PROVIDING it is 
// not sold for profit without the authors written consent, and 
// providing that this notice and the authors name is included. If 
// the source code in  this file is used in any commercial application 
// then acknowledgement must be made to the author of this file 
// (in whatever form you wish).
//
// This file is provided "as is" with no expressed or implied warranty.
// The author accepts no liability for any damage/loss of business that
// this product may cause.
//
// Expect bugs!
// 
// Please use and enjoy. Please let me know of any bugs/mods/improvements 
// that you have found/implemented and I will fix/incorporate them into this
// file. 
//
//-	---------------------------------------------------------------------------
#include "stdafx.h"
#include "InPlaceCheckbox.h"

#include "GridCtrl.h"


#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


/////////////////////////////////////////////////////////////////////////////
// CGridCheckBox

CGridCheckBox::CGridCheckBox()
{
}

CGridCheckBox::~CGridCheckBox()
{
}

// Stoopid win95 accelerator key problem workaround - Matt Weagle.
BOOL CGridCheckBox::PreTranslateMessage(MSG* pMsg) 
{
	// Make sure that the keystrokes continue to the appropriate handlers
	if (pMsg->message == WM_KEYDOWN || pMsg->message == WM_KEYUP)
	{
		::TranslateMessage(pMsg);
		::DispatchMessage(pMsg);
		return TRUE;
	}	

	// Catch the Alt key so we don't choke if focus is going to an owner drawn button
	if (pMsg->message == WM_SYSCHAR)
		return TRUE;

	return CButton::PreTranslateMessage(pMsg);
}

BEGIN_MESSAGE_MAP(CGridCheckBox, CButton)
	//{{AFX_MSG_MAP(CGridCheckBox)
	ON_WM_KILLFOCUS()
	ON_WM_KEYDOWN()
	ON_WM_KEYUP()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CGridCheckBox message handlers

void CGridCheckBox::OnKillFocus(CWnd* pNewWnd) 
{
	CButton::OnKillFocus(pNewWnd);
	
    CInPlaceCheckbox* pOwner = (CInPlaceCheckbox*) GetOwner();  // This MUST be a CInPlaceCheckbox
}

void CGridCheckBox::OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
	if ((nChar == VK_PRIOR || nChar == VK_NEXT ||
		 nChar == VK_DOWN  || nChar == VK_UP   ||
		 nChar == VK_RIGHT || nChar == VK_LEFT) &&
		(GetKeyState(VK_CONTROL) < 0 && GetDlgCtrlID() == IDC_CHECKBOXEDIT))
    {
        CWnd* pOwner = GetOwner();
        if (pOwner)
            pOwner->SendMessage(WM_KEYDOWN, nChar, nRepCnt+ (((DWORD)nFlags)<<16));
        return;
    }

	CButton::OnKeyDown(nChar, nRepCnt, nFlags);
}

void CGridCheckBox::OnKeyUp(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
	if (nChar == VK_ESCAPE) 
	{
        CWnd* pOwner = GetOwner();
        if (pOwner)
            pOwner->SendMessage(WM_KEYUP, nChar, nRepCnt + (((DWORD)nFlags)<<16));
        return;
    }

	if (nChar == VK_TAB || nChar == VK_RETURN || nChar == VK_ESCAPE)
    {
        CWnd* pOwner = GetOwner();
        if (pOwner)
            pOwner->SendMessage(WM_KEYUP, nChar, nRepCnt + (((DWORD)nFlags)<<16));
        return;
    }

	CButton::OnKeyUp(nChar, nRepCnt, nFlags);
}


/////////////////////////////////////////////////////////////////////////////
// CInPlaceCheckbox

CInPlaceCheckbox::CInPlaceCheckbox(CWnd* pParent, CRect& rect, DWORD dwStyle, UINT nID,
                           int nRow, int nColumn, 
						   CString sInitText, 
						   UINT nFirstChar,
                           COLORREF BkColor)
{
    m_BkColor = BkColor;
	m_nNumLines = 4;
	m_sInitText = sInitText;
 	m_nRow		= nRow;
 	m_nCol      = nColumn;
 	m_nLastChar = 0; 
	m_bExitOnArrows = FALSE; //(nFirstChar != VK_LBUTTON);	// If mouse click brought us here,

	// Create the checkbox
 	DWORD ButtonStyle = WS_BORDER|WS_CHILD|WS_VISIBLE|dwStyle;

	int nHeight = rect.Height();
	if (!Create(m_sInitText, ButtonStyle, rect, pParent, nID)) return;

	// Get the maximum width of the text strings
	int nMaxLength = 0;
	CClientDC dc(GetParent());
	CFont* pOldFont = dc.SelectObject(pParent->GetFont());

	nMaxLength += (::GetSystemMetrics(SM_CXVSCROLL) + dc.GetTextExtent(_T(" ")).cx*2);
	dc.SelectObject(pOldFont);

    if (nMaxLength > rect.Width())
	    rect.right = rect.left + nMaxLength;

	// Resize the window;
	MoveWindow(rect);

	SetFont(pParent->GetFont());

	// Set the initial text to m_sInitText
	SetWindowText(m_sInitText);

    // Subclass the control
    m_Checkbox.SubclassDlgItem(IDC_CHECKBOXEDIT, this);
    SetFocus();
    SendMessage(WM_CHAR, nFirstChar);

    //Create the background brush that paints the background;
    m_BackGndBrush.CreateSolidBrush(BkColor);
}

CInPlaceCheckbox::~CInPlaceCheckbox()
{
    m_BackGndBrush.DeleteObject();
}

void CInPlaceCheckbox::EndEdit()
{
    CString str;
    GetWindowText(str);
 
    // Send Notification to parent
    GV_DISPINFO dispinfo;

    dispinfo.hdr.hwndFrom = GetSafeHwnd();
    dispinfo.hdr.idFrom   = GetDlgCtrlID();
    dispinfo.hdr.code     = GVN_ENDLABELEDIT;
 
    dispinfo.item.mask    = LVIF_TEXT|LVIF_PARAM;
    dispinfo.item.row     = m_nRow;
    dispinfo.item.col     = m_nCol;
    dispinfo.item.szText  = str;
    dispinfo.item.lParam  = (LPARAM) m_nLastChar; 
 
    CWnd* pOwner = GetOwner();
    if (IsWindow(pOwner->GetSafeHwnd()))
        pOwner->SendMessage(WM_NOTIFY, GetDlgCtrlID(), (LPARAM)&dispinfo );
 
    // Close this window (PostNcDestroy will delete this)
    PostMessage(WM_CLOSE, 0, 0);
}

void CInPlaceCheckbox::PostNcDestroy() 
{
	CButton::PostNcDestroy();

	delete this;
}

BEGIN_MESSAGE_MAP(CInPlaceCheckbox, CButton)
	//{{AFX_MSG_MAP(CInPlaceCheckbox)
	ON_WM_KILLFOCUS()
	ON_WM_KEYDOWN()
	ON_WM_KEYUP()
	ON_CONTROL_REFLECT(BN_CLICKED, OnClicked)
	ON_WM_CTLCOLOR_REFLECT()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// CInPlaceCheckbox message handlers

void CInPlaceCheckbox::OnKillFocus(CWnd* pNewWnd) 
{
	CButton::OnKillFocus(pNewWnd);

	if (GetSafeHwnd() == pNewWnd->GetSafeHwnd())
        return;
}

void CInPlaceCheckbox::OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
	if ((nChar == VK_PRIOR || nChar == VK_NEXT ||
		 nChar == VK_DOWN  || nChar == VK_UP   ||
		 nChar == VK_RIGHT || nChar == VK_LEFT) &&
		(m_bExitOnArrows || GetKeyState(VK_CONTROL) < 0))
	{
		m_nLastChar = nChar;
		GetParent()->SetFocus();
		return;
	}

	CButton::OnKeyDown(nChar, nRepCnt, nFlags);
}

// Need to keep a lookout for Tabs, Esc and Returns.
void CInPlaceCheckbox::OnKeyUp(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
	if (nChar == VK_ESCAPE) 
		SetWindowText(m_sInitText);	// restore previous text

	if (nChar == VK_TAB || nChar == VK_RETURN || nChar == VK_ESCAPE)
	{
		m_nLastChar = nChar;
		GetParent()->SetFocus();	// This will destroy this window
		return;
	}

	CButton::OnKeyUp(nChar, nRepCnt, nFlags);
}

void CInPlaceCheckbox::SetCheckState(BOOLEAN NewState)
{
    SetCheck(NewState);
}

BOOLEAN CInPlaceCheckbox::GetCheckState()
{
    return GetCheck();
}

void CInPlaceCheckbox::OnClicked() 
{
    //Check or uncheck the box;
    SetCheck(!GetCheck());
}

HBRUSH CInPlaceCheckbox::CtlColor(CDC* pDC, UINT nCtlColor) 
{
  	if (nCtlColor == CTLCOLOR_STATIC)
    {
   		pDC->SetBkMode(TRANSPARENT);
     	return m_BackGndBrush;
  	} 

	HBRUSH hbr = OnCtlColor(pDC, this, nCtlColor);
	return hbr;
}

void CInPlaceCheckbox::ChangePosition(const CRect NewRect)
{
    if (::IsWindow(GetSafeHwnd()))
        MoveWindow(NewRect);
}
